@@ -3,10 +3,10 @@ App::DBBrowser::Join_Union;
 
 use warnings;
 use strict;
-use 5.010000;
+use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.049';
+our $VERSION = '0.990';
 
 use Clone                  qw( clone );
 use List::MoreUtils        qw( any );
@@ -18,8 +18,6 @@ use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::DBBrowser::DB;
 
-sub CLEAR_SCREEN () { "\e[H\e[J" }
-
 
 
 sub new {
@@ -34,9 +32,7 @@ sub __union_tables {
     my $u = $data->{$db}{$schema};
     if ( ! defined $u->{col_names} || ! defined $u->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        my ( $col_names, $col_types ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $u->{tables} );
-        $u->{col_names} = $col_names;
-        $u->{col_types} = $col_types;
+        ( $u->{col_names}, $u->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $u->{tables} );
     }
     my $union = {
         unused_tables => [ map { "- $_" } @{$u->{tables}} ],
@@ -44,25 +40,28 @@ sub __union_tables {
         used_cols     => {},
         saved_cols    => [],
     };
+    my $enough_tables = '  Enough TABLES';
+    my $all_tables    = '  All Tables';
+    my $info          = '  INFO';
+    my @pre_tbl  = ( undef, $enough_tables );
+    my @post_tbl = ( $all_tables, $info );
 
     UNION_TABLE: while ( 1 ) {
-        my $enough_tables = '  Enough TABLES';
-        my $all_tables    = '  All Tables';
-        my @pre_tbl  = ( undef, $enough_tables );
-        my @post_tbl = ( $all_tables, $self->{info}{_info} );
+
         my $choices = [ @pre_tbl, map( "+ $_", @{$union->{used_tables}} ), @{$union->{unused_tables}}, @post_tbl ];
         $self->__print_union_statement( $union );
+        my $prompt = $self->{union_all} ? 'One UNION table for cols:' : 'Choose UNION table:';
         # Choose
         my $idx_tbl = $no_lyt->choose(
             $choices,
-            { %{$self->{info}{lyt_stmt_v}}, prompt => 'Choose UNION table:', index => 1 }
+            { %{$self->{info}{lyt_stmt_v}}, prompt => $prompt, index => 1 }
         );
         return if ! defined $idx_tbl;
         my $union_table = $choices->[$idx_tbl];
         return  if ! defined $union_table;
-        if ( $union_table eq $self->{info}{_info} ) {
+        if ( $union_table eq $info ) {
             if ( ! defined $u->{tables_info} ) {
-                $u->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
+                $u->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $u );
             }
             my $tbls_info = $self->__print_tables_info( $u );
             # Choose
@@ -100,12 +99,12 @@ sub __union_tables {
             push @{$union->{used_tables}}, $union_table;
             $self->{idx_reset_used_tables} = -1;
         }
+        my ( $all_cols, $privious_cols, $void ) = ( q['*'], q['^'], q[' '] );
+        my @short_cuts = ( ( @{$union->{saved_cols}} ? $privious_cols : $void ), $all_cols );
+        my @pre_col = ( $self->{info}{ok}, @short_cuts );
+        unshift @pre_col, undef if $self->{opt}{sssc_mode};
 
         UNION_COLUMN: while ( 1 ) {
-            my ( $all_cols, $privious_cols, $void ) = ( q['*'], q['^'], q[' '] );
-            my @short_cuts = ( ( @{$union->{saved_cols}} ? $privious_cols : $void ), $all_cols );
-            my @pre_col = ( $self->{info}{ok}, @short_cuts );
-            unshift @pre_col, undef if $self->{opt}{sssc_mode};
             my $choices = [ @pre_col, @{$u->{col_names}{$union_table}} ];
             $self->__print_union_statement( $union );
             # Choose
@@ -188,7 +187,8 @@ sub __union_tables {
         $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_ALL_TABLES' );
     }
     else {
-        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( join '_', @{$union->{used_tables}} );
+        #$union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( join '_', @{$union->{used_tables}} );
+        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_SELECTED_TABLES' );
     }
     return $union;
 }
@@ -198,63 +198,79 @@ sub __print_union_statement {
     my ( $self, $union ) = @_;
     my $str;
     if ( $self->{union_all} ) {
-        $str = 'UNION ALL TABLES' . "\n" . 'Col: ';
+        $str = 'UNION ALL TABLES';
         if ( @{$union->{used_tables}} ) {
+            $str .= "\n" . 'Cols: ';
             my $table = $union->{used_tables}[0];
-            $str .= join( ', ', @{$union->{used_cols}{$table}} ) if @{$union->{used_cols}{$table}//[]};
+            if ( defined $union->{used_cols}{$table} && @{$union->{used_cols}{$table}} ) { #
+                $str .= join( ', ', @{$union->{used_cols}{$table}} );
+            }
         }
         $str .= "\n";
     }
     else {
         $str = "SELECT * FROM (\n";
-        my $c = 0;
-        for my $table ( @{$union->{used_tables}} ) {
-            ++$c;
-            $str .= "  SELECT ";
-            $str .= @{$union->{used_cols}{$table}//[]} ? join( ', ', @{$union->{used_cols}{$table}} ) : '?';
-            $str .= " FROM $table";
-            $str .= $c < @{$union->{used_tables}} ? " UNION ALL\n" : "\n";
-        }
         if ( @{$union->{used_tables}} ) {
+            my $c = 0;
+            for my $table ( @{$union->{used_tables}} ) {
+                ++$c;
+                $str .= "  SELECT ";
+                if ( defined $union->{used_cols}{$table} && @{$union->{used_cols}{$table}} ) { #
+                    $str .= join( ', ', @{$union->{used_cols}{$table}} );
+                }
+                else {
+                    $str .= '?';
+                }
+                $str .= " FROM $table";
+                $str .= $c < @{$union->{used_tables}} ? " UNION ALL\n" : "\n";
+            }
             $str .= ") AS ";
-            $str .= $self->{union_all} ? 'UNION_ALL_TABLES' : join '_', @{$union->{used_tables}};
+            #$str .= join '_', @{$union->{used_tables}};
+            $str .= 'Selected_Tables';
             $str .= " \n";
         }
     }
     $str .= "\n";
     my $line_fold = Text::LineFold->new( %{$self->{info}{line_fold}}, ColMax => ( term_size() )[0] - 2 );
-    print CLEAR_SCREEN;
+    print $self->{info}{clear_screen};
     print $line_fold->fold( '', ' ' x $self->{info}{stmt_init_tab}, $str );
 }
 
 
 sub __get_tables_info {
-    my ( $self, $dbh, $db, $schema, $data ) = @_;
+    my ( $self, $dbh, $db, $schema, $u_or_j ) = @_;
     my $tables_info = {};
     my $sth;
     my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $data->{$db}{$schema}{tables} );
-    for my $table ( @{$data->{$db}{$schema}{tables}} ) {
+    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $u_or_j->{tables} );
+    for my $table ( @{$u_or_j->{tables}} ) {
         push @{$tables_info->{$table}}, [ 'Table: ', '== ' . $table . ' ==' ];
-        push @{$tables_info->{$table}}, [
-            'Columns: ',
-            join( ' | ', map {
-                    lc( $data->{$db}{$schema}{col_types}{$table}[$_] )
-                . ' ' . $data->{$db}{$schema}{col_names}{$table}[$_] } 0 .. $#{$data->{$db}{$schema}{col_names}{$table}} )
-        ];
-        if ( @{$pk->{$table}} ) {
+        if ( defined $u_or_j->{col_names} ) {
+            push @{$tables_info->{$table}}, [
+                'Columns: ',
+                join( ' | ', map {
+                        lc( $u_or_j->{col_types}{$table}[$_] )
+                    . ' ' . $u_or_j->{col_names}{$table}[$_] } 0 .. $#{$u_or_j->{col_names}{$table}} )
+            ];
+        }
+        if ( defined $pk && @{$pk->{$table}} ) {
             push @{$tables_info->{$table}}, [ 'PK: ', 'primary key (' . join( ',', @{$pk->{$table}} ) . ')' ];
         }
-        for my $fk_name ( sort keys %{$fk->{$table}} ) {
-            if ( $fk->{$table}{$fk_name} ) {
-                push @{$tables_info->{$table}}, [
-                    'FK: ',
-                    'foreign key (' . join( ',', @{$fk->{$table}{$fk_name}{foreign_key_col}} ) .
-                    ') references ' . $fk->{$table}{$fk_name}{reference_table} .
-                    '(' . join( ',', @{$fk->{$table}{$fk_name}{reference_key_col}} ) .')'
-                ];
+        if ( defined $fk ) {
+            for my $fk_name ( sort keys %{$fk->{$table}} ) {
+                if ( $fk->{$table}{$fk_name} ) {
+                    push @{$tables_info->{$table}}, [
+                        'FK: ',
+                        'foreign key (' . join( ',', @{$fk->{$table}{$fk_name}{foreign_key_col}} ) .
+                        ') references ' . $fk->{$table}{$fk_name}{reference_table} .
+                        '(' . join( ',', @{$fk->{$table}{$fk_name}{reference_key_col}} ) .')'
+                    ];
+                }
             }
         }
+        if ( @{$tables_info->{$table}} == 1 ) {
+            push @{$tables_info->{$table}}, [ '   No INFO available.' ];
+        }
     }
     return $tables_info;
 }
@@ -288,17 +304,16 @@ sub __join_tables {
     my $j = $data->{$db}{$schema};
     if ( ! defined $j->{col_names} || ! defined $j->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        my ( $col_names, $col_types ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $j->{tables} );
-        $j->{col_names} = $col_names;
-        $j->{col_types} = $col_types;
+        ( $j->{col_names}, $j->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $j->{tables} );
     }
     my @tables = map { "- $_" } @{$j->{tables}};
+    my $info = '  INFO';
+    my @pre = ( undef );
 
     MASTER: while ( 1 ) {
         $self->__print_join_statement( $join->{print}{stmt} );
+        my $choices = [ @pre, @tables, $info ];
         # Choose
-        my @pre = ( undef );
-        my $choices = [ @pre, @tables, $self->{info}{_info} ];
         my $idx = $stmt_v->choose(
             $choices,
             { prompt => 'Choose MASTER table:', index => 1 }
@@ -306,9 +321,9 @@ sub __join_tables {
         return if ! defined $idx;
         my $master = $choices->[$idx];
         return if ! defined $master;
-        if ( $master eq $self->{info}{_info} ) {
+        if ( $master eq $info ) {
             if ( ! defined $j->{tables_info} ) {
-                $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
+                $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j );
             }
             my $tbls_info = $self->__print_tables_info( $j );
             # Choose
@@ -330,14 +345,14 @@ sub __join_tables {
         my $backup_master = clone( $join );
 
         JOIN: while ( 1 ) {
-            my $enough_slaves = '  Enough TABLES';
             my ( $idx, $slave );
             my $backup_join = clone( $join );
+            my $enough_slaves = '  Enough TABLES';
+            my @pre = ( undef, $enough_slaves );
 
             SLAVE: while ( 1 ) {
                 $self->__print_join_statement( $join->{print}{stmt} );
-                my @pre = ( undef, $enough_slaves );
-                my $choices = [ @pre, @{$join->{avail_tables}}, $self->{info}{_info} ];
+                my $choices = [ @pre, @{$join->{avail_tables}}, $info ];
                 # Choose
                 $idx = $stmt_v->choose(
                     $choices,
@@ -362,9 +377,9 @@ sub __join_tables {
                 elsif ( $slave eq $enough_slaves ) {
                     last JOIN;
                 }
-                elsif ( $slave eq $self->{info}{_info} ) {
+                elsif ( $slave eq $info ) {
                     if ( ! defined $j->{tables_info} ) {
-                        $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
+                        $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j );
                     }
                     my $tbls_info = $self->__print_tables_info( $j );
                     # Choose
@@ -393,10 +408,10 @@ sub __join_tables {
                 $avail_fk_cols{ $slave . '.' . $col } = $dbh->quote_identifier( undef, $slave, $col );
             }
             my $AND = '';
+            my @pre = ( undef );
 
             ON: while ( 1 ) {
                 $self->__print_join_statement( $join->{print}{stmt} );
-                my @pre = ( undef );
                 push @pre, $self->{info}{_continue} if $AND;
                 # Choose
                 my $pk_col = $stmt_v->choose(
@@ -477,7 +492,7 @@ sub __print_join_statement {
     $join_stmt_pr =~ s/(?=\sLEFT\sOUTER\sJOIN)/\n\ /g;
     $join_stmt_pr .= "\n\n";
     my $line_fold = Text::LineFold->new( %{$self->{info}{line_fold}}, ColMax => ( term_size() )[0] - 2 );
-    print CLEAR_SCREEN;
+    print $self->{info}{clear_screen};
     print $line_fold->fold( '', ' ' x $self->{info}{stmt_init_tab}, $join_stmt_pr );
 }
 
