@@ -3,24 +3,24 @@ App::DBBrowser::Opt;
 
 use warnings;
 use strict;
-use 5.010000;
+use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.049';
+our $VERSION = '0.990';
 
-use Encode                qw( encode );
-use File::Basename        qw( basename );
+use File::Basename        qw( basename fileparse );
 use File::Spec::Functions qw( catfile );
 use FindBin               qw( $RealBin $RealScript );
 #use Pod::Usage            qw( pod2usage );  # "require"-d
 
 use Clone                  qw( clone );
-use Encode::Locale         qw();
-use JSON                   qw( decode_json );
 use Term::Choose           qw( choose );
 use Term::Choose::Util     qw( insert_sep print_hash choose_a_number choose_a_subset choose_multi choose_dirs );
 use Term::ReadLine::Simple qw();
 
+use App::DBBrowser::DB;
+use App::DBBrowser::Auxil;
+
 
 
 sub new {
@@ -32,13 +32,11 @@ sub new {
 sub defaults {
     my ( $self, @keys ) = @_;
     my $defaults = {
-        db_drivers           => [ 'SQLite', 'mysql', 'Pg' ],
-        ask_host_port_per_db => 1,
-        ask_user_pass_per_db => 1,
-        use_env_dbi_user     => 0,
-        use_env_dbi_pass     => 0,
-        use_env_dbi_host     => 0,
-        use_env_dbi_port     => 0,
+        db_plugins           => [ 'SQLite', 'mysql', 'Pg' ],
+        login_host           => 1,
+        login_port           => 1,
+        login_user           => 1,
+        login_pass           => 1,
         menus_config_memory  => 0,
         menu_sql_memory      => 0,
         menus_db_memory      => 0,
@@ -49,19 +47,18 @@ sub defaults {
         thsd_sep             => ',',
         metadata             => 0,
         max_rows             => 50_000,
-        operators            => [ "REGEXP", " = ", " != ", " < ", " > ", "IS NULL", "IS NOT NULL" ],
+        operators            => [ "REGEXP", "REGEXP_i", " = ", " != ", " < ", " > ", "IS NULL", "IS NOT NULL" ],
         parentheses_w        => 0,
         parentheses_h        => 0,
-        regexp_case          => 0,
+        #regexp_case          => 0,
         keep_header          => 0,
         progress_bar         => 40_000,
         min_col_width        => 30,
         tab_width            => 2,
         undef                => '',
-        debug                => 0,
         binary_string        => 'BNRY',
         input_modes          => [ 'Cols', 'Multirow', 'File' ],
-        row_col_filter       => 0,
+        row_col_filter       => 1,
         csv_read             => 0,
         encoding_csv_file    => 'UTF-8',
         max_files            => 15,
@@ -83,7 +80,7 @@ sub defaults {
             sqlite_unicode             => 1,
             sqlite_see_if_its_a_number => 1,
             binary_filter              => 0,
-            dirs_sqlite_search         => undef,
+            db_search_path             => [ $self->{info}{home_dir} ],
         },
         mysql => {
             user              => undef,
@@ -110,26 +107,24 @@ sub defaults {
 sub __multi_choose {
     my ( $self, $key ) = @_;
     my $multi_choose = {
-       _enchant      => [
-            [ 'menus_config_memory', "- Menus config", [ 'Simple', 'Memory' ] ],
-            [ 'menu_sql_memory',     "- Menu  sql",    [ 'Simple', 'Memory' ] ],
-            [ 'menus_db_memory',     "- Menus db",     [ 'Simple', 'Memory' ] ],
-            [ 'table_expand',        "- Print  Table", [ 'Simple', 'Expand' ] ],
-            [ 'keep_header',         "- Table Header", [ 'Simple', 'Each page' ] ],
+       _menu_memory  => [
+            [ 'menus_config_memory', "- Config Menus", [ 'Simple', 'Memory' ] ],
+            [ 'menu_sql_memory',     "- SQL    Menu",  [ 'Simple', 'Memory' ] ],
+            [ 'menus_db_memory',     "- DB     Menus", [ 'Simple', 'Memory' ] ],
+        ],
+       _table_expand => [
+            [ 'table_expand', "- Print  Table", [ 'Simple', 'Expand'    ] ],
+            [ 'keep_header',  "- Table Header", [ 'Simple', 'Each page' ] ],
         ],
         _parentheses => [
             [ 'parentheses_w', "- Parentheses in WHERE",     [ 'NO', '(YES', 'YES(' ] ],
             [ 'parentheses_h', "- Parentheses in HAVING TO", [ 'NO', '(YES', 'YES(' ] ],
         ],
-        _env_dbi     => [
-            [ 'use_env_dbi_user', "- Use DBI_USER", [ 'NO', 'YES' ] ],
-            [ 'use_env_dbi_pass', "- Use DBI_PASS", [ 'NO', 'YES' ] ],
-            [ 'use_env_dbi_host', "- Use DBI_HOST", [ 'NO', 'YES' ] ],
-            [ 'use_env_dbi_port', "- Use DBI_PORT", [ 'NO', 'YES' ] ],
-        ],
         _db_connect  => [
-            [ 'ask_host_port_per_db', "- Ask host/port per DB", [ 'NO', 'YES' ] ],
-            [ 'ask_user_pass_per_db', "- Ask user/pass per DB", [ 'NO', 'YES' ] ],
+            [ 'login_host', "- Host",     [ 'Ask', 'Use DBI_HOST', 'Don\'t set' ] ],
+            [ 'login_port', "- Port",     [ 'Ask', 'Use DBI_PORT', 'Don\'t set' ] ],
+            [ 'login_user', "- User",     [ 'Ask', 'Use DBI_USER' ] ],
+            [ 'login_pass', "- Password", [ 'Ask', 'Use DBI_PASS' ] ],
         ],
         _options_csv => [
             [ 'allow_loose_escapes', "- allow_loose_escapes", [ 'NO', 'YES' ] ],
@@ -152,7 +147,7 @@ sub __menus {
             [ 'config_output',   "- Output" ],
             [ 'config_menu',     "- Menu" ],
             [ 'config_sql',      "- SQL" ],
-            [ 'config_database', "- Database" ],
+            [ 'config_database', "- DB" ],
             [ 'config_insert',   "- Insert" ],
         ],
         config_output => [
@@ -160,27 +155,25 @@ sub __menus {
             [ 'progress_bar',  "- ProgressBar" ],
             [ 'tab_width',     "- Tabwidth" ],
             [ 'undef',         "- Undef" ],
-            [ 'debug',         "- Debug" ],
         ],
         config_menu => [
-            [ '_enchant',  "- Enchant" ],
-            [ 'lock_stmt', "- Lock" ],
-            [ 'mouse',     "- Mouse Mode" ],
-            [ 'sssc_mode', "- Sssc Mode" ],
+            [ '_menu_memory',  "- Menu Memory" ],
+            [ '_table_expand', "- Table Expand" ],
+            [ 'lock_stmt',     "- Lock" ],
+            [ 'mouse',         "- Mouse Mode" ],
+            [ 'sssc_mode',     "- Sssc Mode" ],
         ],
         config_sql => [
             [ 'max_rows',     "- Max Rows" ],
             [ 'metadata',     "- Metadata" ],
             [ 'operators',    "- Operators" ],
             [ '_parentheses', "- Parentheses" ],
-            [ 'regexp_case',  "- Regexp Case" ],
 
         ],
         config_database => [
-            [ '_db_defaults', "- DB Defaults" ],
-            [ 'db_drivers',   "- DB Drivers" ],
-            [ '_db_connect',  "- DB Login" ],
-            [ '_env_dbi',     "- ENV DBI" ],
+            [ 'db_plugins',   "- DB Plugins" ],
+            [ '_db_defaults', "- DB Settings" ],
+            [ '_db_connect',  "- DB Login Mode" ],
         ],
         config_insert => [
             [ 'input_modes',       "- Input modes" ],
@@ -302,12 +295,12 @@ sub set_options {
     my $old_idx = 0;
 
     GROUP: while ( 1 ) {
-        my @pre = ( undef, $self->{info}{$group eq 'main' ? '_continue' : '_confirm'} );
+        my @pre = ( undef, $group eq 'main' ? $self->{info}{_continue} : $self->{info}{_confirm} );
         my $menu = $self->__menus( $group );
         my @real = map( $_->[1], @$menu );
 
         OPTION: while ( 1 ) {
-            my $back = $self->{info}{$group eq 'main' ? '_exit' : '_back'};
+            my $back = $group eq 'main' ? $self->{info}{_quit} : $self->{info}{_back};
             # Choose
             my $idx = choose(
                 [ @pre, @real ],
@@ -392,11 +385,6 @@ sub set_options {
                 my $prompt = 'Print replacement for undefined table vales';
                 $self->__opt_readline( $key, $prompt );
             }
-            elsif ( $key eq 'debug' ) {
-                my $list = $no_yes;
-                my $prompt = 'Debug';
-                $self->__opt_choose_index( $key, $prompt, $list );
-            }
             elsif ( $key eq 'progress_bar' ) {
                 my $digits = 7;
                 my $prompt = '"Threshold ProgressBar"';
@@ -417,11 +405,6 @@ sub set_options {
                 my $prompt = 'Enable Metadata';
                 $self->__opt_choose_index( $key, $prompt, $list );
             }
-            elsif ( $key eq 'regexp_case' ) {
-                my $list = $no_yes;
-                my $prompt = 'REGEXP case sensitiv';
-                $self->__opt_choose_index( $key, $prompt, $list );
-            }
             elsif ( $key eq '_parentheses' ) {
                 my $sub_menu = $self->__multi_choose( $key );
                 $self->__opt_choose_multi( $sub_menu );
@@ -430,10 +413,6 @@ sub set_options {
                 my $sub_menu = $self->__multi_choose( $key );
                 $self->__opt_choose_multi( $sub_menu );
             }
-            elsif ( $key eq '_env_dbi' ) {
-                my $sub_menu = $self->__multi_choose( $key );
-                $self->__opt_choose_multi( $sub_menu );
-            }
             elsif ( $key eq '_db_defaults' ) {
                 $self->database_setting();
             }
@@ -446,16 +425,23 @@ sub set_options {
                 my $available = $self->{info}{avail_operators};
                 $self->__opt_choose_a_list( $key, $available );
             }
-            elsif ( $key eq 'db_drivers' ) {
-                my $available = $self->{info}{avail_db_drivers};
-                $self->__opt_choose_a_list( $key, $available );
+            elsif ( $key eq 'db_plugins' ) {
+                my %installed_db_driver;
+                for my $dir ( @INC ) {
+                    map { $installed_db_driver{( fileparse $_, '.pm' )[0]}++ } glob "$dir/App/DBBrowser/DB/*.pm";
+                }
+                $self->__opt_choose_a_list( $key, [ sort keys %installed_db_driver ] );
             }
             elsif ( $key eq 'mouse' ) {
                 my $max = 4;
                 my $prompt = 'Mouse mode';
                 $self->__opt_number( $key, $prompt, $max );
             }
-            elsif ( $key eq '_enchant' ) {
+            elsif ( $key eq '_menu_memory' ) {
+                my $sub_menu = $self->__multi_choose( $key );
+                $self->__opt_choose_multi( $sub_menu );
+            }
+            elsif ( $key eq '_table_expand' ) {
                 my $sub_menu = $self->__multi_choose( $key );
                 $self->__opt_choose_multi( $sub_menu );
             }
@@ -489,10 +475,10 @@ sub __opt_choose_index {
 }
 
 sub __opt_choose_a_list {
-    my ( $self, $key, $available ) = @_;
+    my ( $self, $key, $available, $index ) = @_;
     my $current = $self->{opt}{$key};
     # Choose_list
-    my $list = choose_a_subset( $available, { current => $current } );
+    my $list = choose_a_subset( $available, { current => $current, index => $index } );
     return if ! defined $list;
     return if ! @$list;
     $self->{opt}{$key} = $list;
@@ -541,31 +527,35 @@ sub __opt_readline {
 
 sub database_setting {
     my ( $self, $db ) = @_;
-    my ( $db_driver, $section );
-    if ( ! defined $db ) {
-        if ( @{$self->{opt}{db_drivers}} == 1 ) {
-            $db_driver = $self->{opt}{db_drivers}[0];
+    my ( $db_driver, $db_plugin, $section );
+    if ( defined $db ) {
+        $db_plugin = $self->{info}{db_plugin};
+        $db_driver = $self->{info}{db_driver};
+        $section   = $db_plugin . '_' . $db;
+        for my $key ( keys %{$self->{opt}{$db_driver}} ) {
+            next if $key eq 'db_search_path';
+            if ( ! defined $self->{opt}{$section}{$key} ) {
+                $self->{opt}{$section}{$key} = $self->{opt}{$db_plugin}{$key};
+            }
+        }
+    }
+    else {
+        if ( @{$self->{opt}{db_plugins}} == 1 ) {
+            $db_plugin = $self->{opt}{db_plugins}[0];
         }
         else {
             # Choose
-            $db_driver = choose(
-                [ undef, @{$self->{opt}{db_drivers}} ],
+            $db_plugin = choose(
+                [ undef, @{$self->{opt}{db_plugins}} ],
                 { %{$self->{info}{lyt_1}} }
             );
-            return if ! defined $db_driver;
+            return if ! defined $db_plugin;
         }
-        $section = $db_driver;
+        $self->{info}{db_plugin} = $db_plugin;
+        my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
+        $db_driver = $obj_db->db_driver();
+        $section = $db_plugin;
     }
-    else {
-        $db_driver = $self->{info}{db_driver};
-        $section   = $db_driver . '_' . $db;
-        for my $key ( keys %{$self->{opt}{$db_driver}} ) {
-            next if $key =~ /^(?:host|port|user)\z/; #
-            next if $key eq 'dirs_sqlite_search';
-            $self->{opt}{$section}{$key} //= $self->{opt}{$db_driver}{$key};
-        }
-    }
-
     my $orig = clone( $self->{opt} );
     my $menus = {
         SQLite => [
@@ -579,14 +569,12 @@ sub database_setting {
             [ 'pg_enable_utf8', "- Enable utf8" ],
         ],
     };
-    if ( $db_driver =~ /^(?:mysql|Pg)\z/ ) {
-        unshift @{$menus->{$db_driver}}, [ 'host', "- Host" ], [ 'port', "- Port" ] if $self->{opt}{ask_host_port_per_db};
-        unshift @{$menus->{$db_driver}}, [ 'user', "- User" ];
+    if ( $db_driver ne 'SQLite' ) {
+        unshift @{$menus->{$db_driver}}, ( [ 'host', "- Host" ], [ 'port', "- Port" ], [ 'user', "- User" ] );
     }
-    if ( ! $db && $db_driver eq 'SQLite' ) {
-        push @{$menus->{$db_driver}}, [ 'dirs_sqlite_search', "- Default DB dirs" ];
-    }
-    push @{$menus->{$db_driver}}, [ 'binary_filter', "- Binary Filter" ], [ '_reset', "  RESET" ];
+    push @{$menus->{$db_driver}}, [ 'binary_filter',  "- Binary Filter" ];
+    push @{$menus->{$db_driver}}, [ 'db_search_path', "  Search directories" ] if ! $db && $db_driver eq 'SQLite';
+    push @{$menus->{$db_driver}}, [ '_reset',         "  RESET" ];
     my $prompt;
     if ( defined $db ) {
         $prompt = 'DB: "' . ( $db_driver eq 'SQLite' ? basename( $db ) : $db ) . '"';
@@ -619,24 +607,23 @@ sub database_setting {
                 $old_idx = $idx;
             }
         }
-
         if ( $key eq '_reset' ) {
             if ( $db ) {
                 delete $self->{opt}{$section};
             }
             else {
-                my @dbs = ();
+                my @databases = ();
                 for my $section ( keys %{$self->{opt}} ) {
-                    push @dbs, $1 if $section =~ /^\Q$db_driver\E_(.+)\z/;
+                    push @databases, $1 if $section =~ /^\Q$db_driver\E_(.+)\z/;
                 }
-                my $dlt = choose_a_subset( [ '*' . $db_driver, sort @dbs ], { p_new => 'Reset: ' } );
-                next DB_OPTION if ! defined $dlt;
-                next DB_OPTION if ! defined $dlt->[0];
-                for my $db ( @$dlt ) {
-                    if ( $db eq '*' . $db_driver ) {
+                my $choices = choose_a_subset( [ '*' . $db_driver, sort @databases ], { p_new => 'Reset: ' } );
+                next DB_OPTION if ! $choices->[0];
+                for my $item ( @$choices ) {
+                    if ( $item eq '*' . $db_driver ) {
                         $self->{opt}{$db_driver} = $self->defaults( $db_driver );
                     }
                     else {
+                        my $db = $item;
                         my $section = $db_driver . '_' . $db;
                         delete $self->{opt}{$section};
                     }
@@ -668,7 +655,7 @@ sub database_setting {
                 my $prompt = 'Enable Binary Filter';
                 $self->__db_opt_choose_index( $section, $key, $prompt, $no_yes );
             }
-            elsif ( $key eq 'dirs_sqlite_search' ) {
+            elsif ( $key eq 'db_search_path' ) {
                 $self->__db_opt_choose_dirs( $section, $key, $prompt );
             }
             else { die "Unknown key: $key" }
@@ -767,17 +754,17 @@ sub __db_opt_readline {
 
 sub __write_config_files {
     my ( $self ) = @_;
-    my $regexp_db_drivers = join '|', map quotemeta, @{$self->defaults( qw( db_drivers ) )};
+    my $regexp_db_plugins = join '|', map quotemeta, @{$self->defaults( qw( db_plugins ) )};
     my $fmt = $self->{info}{conf_file_fmt};
     my $tmp = {};
     for my $section ( sort keys %{$self->{opt}} ) {
-        if ( $section =~ /^($regexp_db_drivers)(?:_(.+))?\z/ ) {
+        if ( $section =~ /^($regexp_db_plugins)(?:_(.+))?\z/ ) {
             die $section if ref( $self->{opt}{$section} ) ne 'HASH';
-            my ( $db_driver, $conf_sect ) = ( $1, $2 );
-            $conf_sect //= '*' . $db_driver;
+            my ( $db_plugin, $conf_sect ) = ( $1, $2 );
+            $conf_sect = '*' . $db_plugin if ! defined $conf_sect;
             for my $key ( keys %{$self->{opt}{$section}} ) {
                 next if $key =~ /^_/;
-                $tmp->{$db_driver}{$conf_sect}{$key} = $self->{opt}{$section}{$key};
+                $tmp->{$db_plugin}{$conf_sect}{$key} = $self->{opt}{$section}{$key};
             }
         }
         else {
@@ -788,10 +775,10 @@ sub __write_config_files {
             $tmp->{$generic}{$key} = $self->{opt}{$key};
         }
     }
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
     for my $name ( keys %$tmp ) {
-        $self->write_json( sprintf( $fmt, $name ), $tmp->{$name}  );
+        $auxil->write_json( sprintf( $fmt, $name ), $tmp->{$name}  );
     }
-
 }
 
 
@@ -799,21 +786,22 @@ sub read_config_files {
     my ( $self ) = @_;
     $self->{opt} = $self->defaults();
     my $fmt = $self->{info}{conf_file_fmt};
-    for my $db_driver ( @{$self->defaults( qw( db_drivers ) )} ) {
-        my $file = sprintf( $fmt, $db_driver );
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
+    for my $db_plugin ( @{$self->defaults( qw( db_plugins ) )} ) {
+        my $file = sprintf( $fmt, $db_plugin );
         if ( -f $file && -s $file ) {
-            my $tmp = $self->read_json( $file );
+            my $tmp = $auxil->read_json( $file );
             for my $conf_sect ( keys %$tmp ) {
-                my $section = $db_driver . ( $conf_sect =~ /^\*(?:$db_driver)\z/ ? '' : '_' . $conf_sect );
+                my $section = $db_plugin . ( $conf_sect =~ /^\*(?:$db_plugin)\z/ ? '' : '_' . $conf_sect );
                 for my $key ( keys %{$tmp->{$conf_sect}} ) {
-                    $self->{opt}{$section}{$key} = $tmp->{$conf_sect}{$key} if exists $self->{opt}{$db_driver}{$key};
+                    $self->{opt}{$section}{$key} = $tmp->{$conf_sect}{$key} if exists $self->{opt}{$db_plugin}{$key};
                 }
             }
         }
     }
     my $file =  sprintf( $fmt, $self->{info}{sect_generic} );
     if ( -f $file && -s $file ) {
-        my $tmp = $self->read_json( $file );
+        my $tmp = $auxil->read_json( $file );
         for my $key ( keys %$tmp ) {
             $self->{opt}{$key} = $tmp->{$key} if exists $self->{opt}{$key};
         }
@@ -822,30 +810,6 @@ sub read_config_files {
 }
 
 
-sub write_json {
-    my ( $self, $file, $h_ref ) = @_;
-    my $json = JSON::XS->new->utf8( 1 )->pretty->canonical->encode( $h_ref );
-    open my $fh, '>', encode( 'locale_fs', $file ) or die $!;
-    print $fh $json;
-    close $fh;
-}
-
-
-sub read_json {
-    my ( $self, $file ) = @_;
-    return {} if ! -f encode( 'locale_fs', $file );
-    open my $fh, '<', encode( 'locale_fs', $file ) or die $!;
-    my $json = do { local $/; <$fh> };
-    close $fh;
-    my $h_ref = {};
-    if ( ! eval {
-        $h_ref = decode_json( $json ) if $json;
-        1 }
-    ) {
-        die "In '$file':\n$@";
-    }
-    return $h_ref;
-}
 
 
 1;
