@@ -3,10 +3,10 @@ App::DBBrowser::Table;
 
 use warnings;
 use strict;
-use 5.010000;
+use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.049';
+our $VERSION = '0.990';
 
 use Clone                  qw( clone );
 use List::MoreUtils        qw( any first_index );
@@ -19,9 +19,7 @@ use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::DBBrowser::DB;
 #use App::DBBrowser::Table::Insert;  # "require"-d
-use App::DBBrowser::Util;
-
-sub CLEAR_SCREEN () { "\e[H\e[J" }
+use App::DBBrowser::Auxil;
 
 
 
@@ -33,7 +31,7 @@ sub new {
 
 sub __on_table {
     my ( $self, $sql, $dbh, $table, $select_from_stmt, $qt_columns, $pr_columns ) = @_;
-    my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
     my $sub_stmts = {
         Select => [ qw( print_table columns aggregate distinct where group_by having order_by limit lock ) ],
@@ -67,14 +65,14 @@ sub __on_table {
     );
     my ( $DISTINCT, $ALL, $ASC, $DESC, $AND, $OR ) = ( "DISTINCT", "ALL", "ASC", "DESC", "AND", "OR" );
     if ( $self->{info}{lock} == 0 ) {
-        $util->__reset_sql( $sql );
+        $auxil->__reset_sql( $sql );
     }
     my $backup_sql = clone( $sql );
     my $old_idx = 1;
 
     CUSTOMIZE: while ( 1 ) {
         $backup_sql = clone( $sql ) if $sql_type eq 'Select';
-        $util->__print_sql_statement( $sql, $table, $sql_type );
+        $auxil->__print_sql_statement( $sql, $table, $sql_type );
         my $choices = [ $customize{hidden}, undef, @customize{@{$sub_stmts->{$sql_type}}} ];
         # Choose
         my $idx = $stmt_h->choose(
@@ -88,7 +86,7 @@ sub __on_table {
             }
             else {
                 if ( $sql->{print}{where_stmt} || $sql->{print}{set_stmt} ) {
-                    $util->__reset_sql( $sql );
+                    $auxil->__reset_sql( $sql );
                     next CUSTOMIZE;
                 }
                 else {
@@ -113,7 +111,7 @@ sub __on_table {
             if ( $self->{info}{lock} == 1 ) {
                 $self->{info}{lock} = 0;
                 $customize{lock} = $lk->[0];
-                $util->__reset_sql( $sql );
+                $auxil->__reset_sql( $sql );
             }
             elsif ( $self->{info}{lock} == 0 )   {
                 $self->{info}{lock} = 1;
@@ -127,7 +125,7 @@ sub __on_table {
         }
         elsif ( $custom eq $customize{'columns'} ) {
             if ( ! ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) ) {
-                $util->__reset_sql( $sql );
+                $auxil->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             $sql->{quote}{chosen_cols} = [];
@@ -138,7 +136,7 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @print_col = $stmt_h->choose(
                     $choices,
@@ -182,7 +180,7 @@ sub __on_table {
             DISTINCT: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, $DISTINCT, $ALL ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $select_distinct = $stmt_h->choose(
                     $choices
@@ -207,7 +205,7 @@ sub __on_table {
         }
         elsif ( $custom eq $customize{'aggregate'} ) {
             if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
-                $util->__reset_sql( $sql );
+                $auxil->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             $sql->{quote}{aggr_cols} = [];
@@ -217,7 +215,7 @@ sub __on_table {
             AGGREGATE: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, @{$self->{info}{avail_aggregate}} ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $aggr = $stmt_h->choose(
                     $choices
@@ -253,7 +251,7 @@ sub __on_table {
                     if ( $aggr eq 'COUNT' ) {
                         my $choices = [ $ALL, $DISTINCT ];
                         unshift @$choices, undef if $self->{opt}{sssc_mode};
-                        $util->__print_sql_statement( $sql, $table, $sql_type );
+                        $auxil->__print_sql_statement( $sql, $table, $sql_type );
                         # Choose
                         my $all_or_distinct = $stmt_h->choose(
                             $choices
@@ -270,7 +268,7 @@ sub __on_table {
                     }
                     my $choices = [ @cols ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     my $print_col = $stmt_h->choose(
                         $choices
@@ -303,7 +301,7 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $print_col = $stmt_h->choose(
                     $choices,
@@ -333,7 +331,7 @@ sub __on_table {
                 ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
                 $sql->{quote}{set_stmt} .= $col_sep . $quote_col . ' =';
                 $sql->{print}{set_stmt} .= $col_sep . $print_col . ' =';
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Readline
                 my $value = $trs->readline( $print_col . ': ' );
                 if ( ! defined $value ) {
@@ -374,7 +372,7 @@ sub __on_table {
                 elsif ( $self->{opt}{parentheses_w} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $print_col = $stmt_h->choose(
                     [ @pre, @choices ]
@@ -409,7 +407,7 @@ sub __on_table {
                 if ( $count > 0 && $sql->{quote}{where_stmt} !~ /\(\z/ ) {
                     my $choices = [ $AND, $OR ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $AND_OR = $stmt_h->choose(
                         $choices
@@ -449,7 +447,7 @@ sub __on_table {
         }
         elsif ( $custom eq $customize{'group_by'} ) {
             if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
-                $util->__reset_sql( $sql );
+                $auxil->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             my $col_sep = ' ';
@@ -463,7 +461,7 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @print_col = $stmt_h->choose(
                     $choices,
@@ -533,7 +531,7 @@ sub __on_table {
                 elsif ( $self->{opt}{parentheses_h} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $aggr = $stmt_h->choose(
                     [ @pre, @choices ]
@@ -568,7 +566,7 @@ sub __on_table {
                 if ( $count > 0 && $sql->{quote}{having_stmt} !~ /\(\z/ ) {
                     my $choices = [ $AND, $OR ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $AND_OR = $stmt_h->choose(
                         $choices
@@ -615,7 +613,7 @@ sub __on_table {
                     $print_aggr                 =           $aggr . "(";
                     my $choices = [ @cols ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $print_col = $stmt_h->choose(
                         $choices
@@ -661,7 +659,7 @@ sub __on_table {
             ORDER_BY: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, @cols ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $print_col = $stmt_h->choose(
                     $choices
@@ -691,7 +689,7 @@ sub __on_table {
                 $sql->{print}{order_by_stmt} .= $col_sep . $print_col;
                 $choices = [ $ASC, $DESC ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $direction = $stmt_h->choose(
                     $choices
@@ -718,7 +716,7 @@ sub __on_table {
             LIMIT: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, $only_limit, $offset_and_limit ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $choice = $stmt_h->choose(
                     $choices
@@ -798,14 +796,15 @@ sub __on_table {
                 }
             }
             my $changed = 0;
+            my @pre = ( undef, $self->{info}{_confirm} );
+            my $prompt = 'Choose:';
+            my $choose_type = 'Your choice:';
 
             HIDDEN: while ( 1 ) {
                 my @cols = $stmt_key eq 'chosen_cols'
                     ? ( @{$sql->{print}{$stmt_key}}                                  )
                     : ( @{$sql->{print}{aggr_cols}}, @{$sql->{print}{group_by_cols}} );
-                my @pre = ( undef, $self->{info}{_confirm} );
-                my $prompt = 'Choose:';
-                my $choose_type = 'Your choice:';
+
                 my $default = 0;
                 if ( $sql_type eq 'Select' ) {
                     unshift @pre, $choose_type;
@@ -813,7 +812,7 @@ sub __on_table {
                     $default = 1;
                 }
                 my $choices = [ @pre, map( "- $_", @cols ) ];
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $i = $stmt_h->choose(
                     $choices,
@@ -839,7 +838,7 @@ sub __on_table {
                         $st =~ s/^-\ //;
                         $sql_type = $st;
                         $old_idx = 1;
-                        $util->__reset_sql( $sql );
+                        $auxil->__reset_sql( $sql );
                     }
                     last HIDDEN;
                 }
@@ -872,7 +871,7 @@ sub __on_table {
                     $changed++;
                     next HIDDEN;
                 }
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $function = $stmt_h->choose(
                     [ undef, map( "  $_", @functions ) ],
@@ -883,9 +882,9 @@ sub __on_table {
                 }
                 $function =~ s/^\s\s//;
                 ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-                my ( $quote_hidd, $print_hidd ) = $obj_db->col_functions( $function, $quote_col, $print_col );
+                my ( $quote_hidd, $print_hidd ) = $self->__col_functions( $function, $quote_col, $print_col );
                 if ( ! defined $quote_hidd ) {
                     next HIDDEN;
                 }
@@ -932,21 +931,21 @@ sub __on_table {
                 push @arguments, $self->{opt}{max_rows};
             }
             local $| = 1;
-            print CLEAR_SCREEN;
-            say 'Database : ...' if $self->{opt}{progress_bar};
+            print $self->{info}{clear_screen};
+            print 'Database : ...' . "\n" if $self->{opt}{progress_bar};
             my $sth = $dbh->prepare( $select );
             $sth->execute( @arguments );
             my $col_names = $sth->{NAME};
             my $all_arrayref = $sth->fetchall_arrayref;
             unshift @$all_arrayref, $col_names;
-            print CLEAR_SCREEN;
+            print $self->{info}{clear_screen};
             return $all_arrayref;
         }
         elsif ( $custom eq $customize{'commit'} ) {
             my ( $qt_table ) = $select_from_stmt =~ /^SELECT\s.*?\sFROM\s(.*)\z/;
             local $| = 1;
-            print CLEAR_SCREEN;
-            say 'Database : ...' if $self->{opt}{progress_bar};
+            print $self->{info}{clear_screen};
+            print 'Database : ...' . "\n" if $self->{opt}{progress_bar};
             my $stmt = $map_sql_types{$sql_type};
             my $to_execute;
             my $transaction;
@@ -984,7 +983,7 @@ sub __on_table {
                     }
                     my $nr_rows   = $sql_type eq 'Insert' ? @$to_execute : $sth->rows;
                     my $commit_ok = sprintf $commit_fmt{$sql_type}, $nr_rows;
-                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
                     my $choices = [ undef,  $commit_ok ];
                     # Choose
                     my $choice = $stmt_h->choose(
@@ -996,14 +995,13 @@ sub __on_table {
                     }
                     else {
                         $dbh->rollback;
-                        $util->__reset_sql( $sql );
+                        $auxil->__reset_sql( $sql );
                         next CUSTOMIZE;
                     }
                     1;
                     }
                 ) {
-                    say 'Commit:';
-                    $util->__print_error_message( "$@rolling back ...\n" );
+                    $auxil->__print_error_message( "$@rolling back ...\n", 'Commit' );
                     eval { $dbh->rollback };#
                 }
             }
@@ -1017,7 +1015,7 @@ sub __on_table {
                     ( $nr_rows ) = $dbh->selectrow_array( $count_stmt, undef, @{$sql->{quote}{where_args}} );
                 }
                 my $commit_ok = sprintf $commit_fmt{$sql_type}, $nr_rows;
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 my $choices = [ undef,  $commit_ok ];
                 # Choose
                 my $choice = $stmt_h->choose(
@@ -1031,7 +1029,7 @@ sub __on_table {
                     }
                 }
                 else {
-                    $util->__reset_sql( $sql );
+                    $auxil->__reset_sql( $sql );
                     next CUSTOMIZE;
                 }
             }
@@ -1060,7 +1058,7 @@ sub __unambiguous_key {
 
 sub __set_operator_sql {
     my ( $self, $sql, $clause, $table, $cols, $qt_columns, $quote_col, $sql_type ) = @_;
-    my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
     my ( $stmt, $args );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
     if ( $clause eq 'where' ) {
@@ -1073,7 +1071,7 @@ sub __set_operator_sql {
     }
     my $choices = [ @{$self->{opt}{operators}} ];
     unshift @$choices, undef if $self->{opt}{sssc_mode};
-    $util->__print_sql_statement( $sql, $table, $sql_type );
+    $auxil->__print_sql_statement( $sql, $table, $sql_type );
     # Choose
     my $operator = $stmt_h->choose(
         $choices
@@ -1087,7 +1085,7 @@ sub __set_operator_sql {
     $operator =~ s/^\s+|\s+\z//g;
     if ( $operator !~ /\s%?col%?\z/ ) {
         my $trs = Term::ReadLine::Simple->new();
-        if ( $operator !~ /REGEXP\z/ ) {
+        if ( $operator !~ /REGEXP(_i)?\z/ ) {
             $sql->{quote}{$stmt} .= ' ' . $operator;
             $sql->{print}{$stmt} .= ' ' . $operator;
         }
@@ -1100,7 +1098,7 @@ sub __set_operator_sql {
             $sql->{print}{$stmt} .= '(';
 
             IN: while ( 1 ) {
-                $util->__print_sql_statement( $sql, $table, $sql_type );
+                $auxil->__print_sql_statement( $sql, $table, $sql_type );
                 # Readline
                 my $value = $trs->readline( 'Value: ' );
                 if ( ! defined $value ) {
@@ -1127,7 +1125,7 @@ sub __set_operator_sql {
             }
         }
         elsif ( $operator =~ /^(?:NOT\s)?BETWEEN\z/ ) {
-            $util->__print_sql_statement( $sql, $table, $sql_type );
+            $auxil->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value_1 = $trs->readline( 'Value: ' );
             if ( ! defined $value_1 ) {
@@ -1139,7 +1137,7 @@ sub __set_operator_sql {
             $sql->{quote}{$stmt} .= ' ' . '?' .      ' AND';
             $sql->{print}{$stmt} .= ' ' . $value_1 . ' AND';
             push @{$sql->{quote}{$args}}, $value_1;
-            $util->__print_sql_statement( $sql, $table, $sql_type );
+            $auxil->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value_2 = $trs->readline( 'Value: ' );
             if ( ! defined $value_2 ) {
@@ -1152,9 +1150,9 @@ sub __set_operator_sql {
             $sql->{print}{$stmt} .= ' ' . $value_2;
             push @{$sql->{quote}{$args}}, $value_2;
         }
-        elsif ( $operator =~ /REGEXP\z/ ) {
+        elsif ( $operator =~ /REGEXP(_i)?\z/ ) {
             $sql->{print}{$stmt} .= ' ' . $operator;
-            $util->__print_sql_statement( $sql, $table, $sql_type );
+            $auxil->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value = $trs->readline( 'Pattern: ' );
             if ( ! defined $value ) {
@@ -1164,18 +1162,16 @@ sub __set_operator_sql {
                 return;
             }
             $value = '^$' if ! length $value;
-            if ( $self->{info}{db_driver} eq 'SQLite' ) {
-                $value = qr/$value/i if ! $self->{opt}{regexp_case};
-                $value = qr/$value/  if   $self->{opt}{regexp_case};
-            }
             $sql->{quote}{$stmt} =~ s/.*\K\s\Q$quote_col\E//;
+            my $do_not_match_regexp = $operator =~ /^NOT/       ? 1 : 0;
+            my $case_sensitive      = $operator =~ /REGEXP_i\z/ ? 0 : 1;
             my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-            $sql->{quote}{$stmt} .= $obj_db->sql_regexp( $quote_col, $operator =~ /^NOT/ ? 1 : 0 );
+            $sql->{quote}{$stmt} .= $obj_db->sql_regexp( $quote_col, $do_not_match_regexp, $case_sensitive );
             $sql->{print}{$stmt} .= ' ' . "'$value'";
             push @{$sql->{quote}{$args}}, $value;
         }
         else {
-            $util->__print_sql_statement( $sql, $table, $sql_type );
+            $auxil->__print_sql_statement( $sql, $table, $sql_type );
             my $prompt = $operator =~ /LIKE\z/ ? 'Pattern: ' : 'Value: ';
             # Readline
             my $value = $trs->readline( $prompt );
@@ -1201,7 +1197,7 @@ sub __set_operator_sql {
         $sql->{print}{$stmt} .= ' ' . $operator;
         my $choices = [ @$cols ];
         unshift @$choices, undef if $self->{opt}{sssc_mode};
-        $util->__print_sql_statement( $sql, $table, $sql_type );
+        $auxil->__print_sql_statement( $sql, $table, $sql_type );
         # Choose
         my $print_col = $stmt_h->choose(
             $choices,
@@ -1238,6 +1234,55 @@ sub __set_operator_sql {
 }
 
 
+sub __col_functions {
+    my ( $self, $func, $quote_col, $print_col ) = @_;
+    my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
+    my $obj_ch = Term::Choose->new();
+    my ( $quote_f, $print_f );
+    $print_f = $self->{info}{hidd_func_pr}{$func} . '(' . $print_col . ')';
+    if ( $func =~ /^Epoch_to_Date(?:Time)?\z/ ) {
+        my $prompt = "$print_f\nInterval:";
+        my ( $microseconds, $milliseconds, $seconds ) = (
+            '  ****************   Micro-Second',
+            '  *************      Milli-Second',
+            '  **********               Second' );
+        my $choices = [ undef, $microseconds, $milliseconds, $seconds ];
+        # Choose
+        my $interval = $obj_ch->choose(
+            $choices,
+            { %{$self->{info}{lyt_stmt_v}}, prompt => $prompt }
+        );
+        return if ! defined $interval;
+        my $div = $interval eq $microseconds ? 1000000 :
+                  $interval eq $milliseconds ? 1000 : 1;
+        if ( $func eq 'Epoch_to_DateTime' ) {
+            $quote_f = $obj_db->epoch_to_datetime( $quote_col, $div );
+        }
+        else {
+            $quote_f = $obj_db->epoch_to_date( $quote_col, $div );
+        }
+    }
+    elsif ( $func eq 'Truncate' ) {
+        my $prompt = "TRUNC $print_col\nDecimal places:";
+        my $choices = [ undef, 0 .. 9 ];
+        # Choose
+        my $precision = $obj_ch->choose(
+            $choices,
+            { %{$self->{info}{lyt_stmt_h}}, prompt => $prompt }
+        );
+        return if ! defined $precision;
+        $quote_f = $obj_db->truncate( $quote_col, $precision );
+    }
+    elsif ( $func eq 'Bit_Length' ) {
+        $quote_f = $obj_db->bit_length( $quote_col );
+    }
+    elsif ( $func eq 'Char_Length' ) {
+        $quote_f = $obj_db->char_length( $quote_col );
+    }
+    return $quote_f, $print_f;
+}
+
+
 
 
 1;
